
                                                 (*:  --Ŀ
                  [http://www.royale.linux-fan.com]ͻ 
                  ĿĿĿĿĿ  Ŀ  |
                .                Ŀ          
                :                   Ĵ  :
                    ٳ   Ŀ ٳ    Ŀ      .
                |           
                 [1999][royale@ifrance.com]ͼ
                --  :*)

(* Unit Graphique Mode13 *)
unit mode13;

(*******************************) INTERFACE (********************************)

type Tpalette=array [0..767] of byte; (* Palette 256 Couleurs Rouge/Vert/Bleu *)
const FADE_SPEED=4;         (* Vitesse de fondu *)
      scrn_pg=ptr($A000,0); (* Pointeur sur la mmoire vido *)
var pal:Tpalette;           (* Palette principale *)

(* Dfinition des routines graphiques *)
procedure setpal;
procedure init_mode13;
procedure close_mode13;
procedure copypage(source,dest:pointer);
procedure clearpage(page:pointer);
procedure copyblock(x,y:word; source:pointer; xs,ys:word; dest:pointer; xd,yd:word; mask:boolean);
procedure putpixel(page:pointer;x,y:word;col:byte);
procedure waitretrace;
procedure outtextxy(page:pointer; x,y:word; mot:string; tcol,fcol:byte);
procedure fade_in;
procedure fade_out;
procedure wdelay(time:word);


(****************************) IMPLEMENTATION (******************************)

procedure fade_in;
(* Ralise un "fondu" d'entr de palette.
   L'cran noir va alors s'claircir pour laisser apparatre l'image. *)
var i,j:word;
    palette:Tpalette;
begin
  palette:=pal;
(* Palette noire *)
  for i:=0 to 767 do pal[i]:=0;
(* Boucle *)
  for j:=0 to 255 div FADE_SPEED do begin
(* Augmenter toutes les couleurs *)
    for i:=0 to 767 do if pal[i]+FADE_SPEED<palette[i] then inc(pal[i],FADE_SPEED) else pal[i]:=palette[i];
(* Synchronisation balayage *)
    waitretrace;
(* Actualiser la palette *)
    setpal;
  end;
end;

procedure fade_out;
(* Ralise un "fondu" de sortie de la palette.
   Effet inverse que le fade_in. *)
var i,j:word;
begin
  for j:=0 to 255 div FADE_SPEED do begin
    for i:=0 to 767 do if pal[i]-FADE_SPEED>0 then dec(pal[i],FADE_SPEED) else pal[i]:=0;
    waitretrace;
    setpal;
  end;
end;

procedure init_mode13; assembler;
(* Initialise le Mode VGA 13h (320x200) *)
asm
  mov ax,13h
  int 10h            (* Interruption graphique, Fonction 0 *)
end;

procedure close_mode13; assembler;
(* Restaure le Mode CGA 3 (80x25) *)
asm
  mov ax,03h
  int 10h            (* Interruption graphique, Fonction 0 *)
end;

procedure copypage(source,dest:pointer); assembler;
(* Copie une page cran virtuelle du pointeur source vers le pointer
   destination. 320x200 points sont dplacs soit 320x200 octets ou
   encore 320x200/4 double-mots (Copie 32bits plus performante !) *)
asm
  push ds            (* Sauver le registre de donnes sur la pile *)
  lds si,source      (* ds:si pointe sur la source *)
  les di,dest        (* es:di pointe sur la destination *)
  mov cx,320*200/4   (* compteur : 320x200/4 *)
  db 66h;rep movsw   (* dplacer cx double-mots de ds:si vers es:di *)
  pop ds             (* Rcuprer le registre sur la pile *)
end;

procedure clearpage(page:pointer); assembler;
(* Efface une page cran virtuelle.
   En fait, on la remplie par la couleur 0 (Noir dans New Entropy) *)
asm
  les di,page        (* es:di pointe sur la page *)
  mov cx,320*200/4   (* compteur : 320x200/4 *)
  db 66h;xor ax,ax   (* couleur  stocker : le double mot 0 *)
  db 66h;rep stosw   (* assigner cx double mot 0 *)
end;

procedure copyblock(x,y:word; source:pointer; xs,ys:word; dest:pointer; xd,yd:word; mask:boolean); assembler;
(* Copie un bloc d'image de x points sur y points.
   Le bloc est copi depuis la page source aux coordonnes xs,ys
   Le bloc est coll dans la page destination aux coordonnes xd,yd
   Si le paramtre Mask vaut FALSE, la couleur de fond (0) n'est pas copie :
   cel permet de copier un "Sprite", une figure de forme quelconque inscrite
   dans le rectangle *)
asm
  push ds            (* Sauver le registre de donnes sur la pile *)

  cmp x,0            (* Si une des dimensions du rectangle est nulle, *)
  je @fin            (* On peut quitter immdiatement ! *)
  cmp y,0
  je @fin

  lds si,source      (* ds:si pointe sur la source *)
  mov ax,ys
  shl ax,6           (* Multiplication rapide par 320 par dcalage de bits : *)
  add si,ax          (* 2^6+2^8=64+256=320 ! *)
  shl ax,2
  add si,ax          (* si <- si+ys*320+xs *)
  add si,xs          (* Ajouter  la source l'offset du bloc *)

  les di,dest        (* es:di pointe sur la destination *)
  mov ax,yd
  shl ax,6
  add di,ax
  shl ax,2
  add di,ax          (* di <- di+yd*320+xd *)
  add di,xd          (* Ajouter  la destination l'offset du bloc *)

  mov bx,320
  sub bx,x           (* Placer dans le registre bx le dcalage de fin de ligne *)
  mov cx,y           (* Placer dans le registre cx le nombre de lignes *)

  cmp mask,false     (* Si le masque est faux, on saute  la sous-procedure *)
  je @sprite         (* Sprite... *)

@block:              (* Block Normal *)
  @boucle1:
    push cx
    mov cx,x
    mov dx,cx
    shr cx,2
    db 66h;rep movsw (* Copier x div 4 double mots *)
    mov cx,dx
    and cx,3
    rep movsb        (* Copier x mod 4 octets (ce qui reste) *)
    add di,bx
    add si,bx        (* Ajouter le dcalage de fin de ligne *)
    pop cx
    dec cx           (* Rpeter cette opration cx=y fois *)
  jnz @boucle1
  jmp @fin

@sprite:             (* Sprite *)
  push cx
  mov cx,x           (* Copie une ligne *)
  @line:
    mov ah,ds:[si]   (* Lire un points dans la source *)
    or ah,ah         (* Est-ce la couleur 0 ? *)
    je @noir         (* Oui -> ne pas la copier *)
    mov es:[di],ah   (* Sinon la copier *)
  @noir:
    inc di
    inc si           (* Point suivant *)
    dec cx
  jnz @line          (* Finir la ligne *)
    add di,bx
    add si,bx
    pop cx
    dec cx
  jnz @sprite        (* Copier cx=y lignes *)

  @fin:
  pop ds             (* Rcuprer le registre sur la pile *)
end;

procedure putpixel(page:pointer;x,y:word;col:byte); assembler;
(* Affiche un point aux coordonnes (x,y) dans une page *)
asm
  les di,page        (* es:di pointe sur la page destination *)
  mov ax,y
  shl ax,6
  mov di,ax
  shl ax,2
  add di,ax
  add di,x           (* di <- di+y*320+x *)
  mov al,col         (* Placer la couleur dans le registre al *)
  mov es:[di],al     (* Copier un point *)
end;

procedure waitretrace; assembler;
(* Attendre que le rayon cathodique passe en haut de l'cran.
   Ceci permet de synchroniser les copies vers la mmoire vido avec le
   rafraichissement afin d'viter les clignotements. *)
asm
  mov dx,3dah
@wait1:
  in al,dx           (* Lire sur le port[$3DA] *)
  test al,08h
  jnz @wait1         (* Jusqu' ce que le bit s'allume *)
@wait2:
  in al,dx           (* Lire sur le port[$3DA] *)
  test al,08h
  jz @wait2          (* Jusqu' ce que le bit s'teigne *)
end;

procedure setpal; assembler;
(* Installe la palette PAL comme palette courante *)
asm
  cld
  xor di,di
  xor bx,bx
  @l1:
    mov dx,03c8h     (* Port[$3C8+n couleur] *)
    mov ax,bx
    out dx,al        (* Envoyer le n de la couleur sur le port *)
    inc dx
    mov cx,3         (* Copier Rouge/Vert/Bleu *)
    @l2:
      mov al,byte ptr pal[di]
      shr al,1
      shr al,1
      out dx,al      (* Envoyer la composante sur le port *)
      inc di
    loop @l2         (* 3 composantes *)
    inc bx
    cmp bx,256
  jne @l1            (* 256 couleurs *)
end;

procedure outtextxy(page:pointer; x,y:word; mot:string; tcol,fcol:byte);
(* Affiche une chaine de caractre dans la page aux coordonnes (x,y) avec la
   couleur TCOL et une ombre de couleur FCOL
   La police utilise a une rsolution de 8x14 points *)
type poltyp=array [0..255,0..13] of byte;
(* Type de la police : Tableau
       256 caractre (0..255)
       14 points de haut (0..13)
       8 points de large (8bits sur le byte) *)
     polptr=^poltyp; (* Pointer de type poltyp*)
const police:polptr=nil; (* Pointeur de police 'constant' initialis vide. *)
var xpol,ypol,poss,temp:byte;
begin
  if police=nil then asm       (* Si la police est vide *)
    push bp                    (* Alors l'initialiser *)
    mov ax,1130h               (* Fonction 11h, Sous-fonction 30h -> Charger *)
    mov bh,2                   (* Police n2 (8x14) *)
    int 10h                    (* Interruption Graphique *)
    mov word ptr [police],bp   (* Offset de la police *)
    mov word ptr [police+2],es (* Segment de la police *)
    pop bp                     (* Maintenant la constante police a change ! *)
  end;
  (* Pour chaque lettre du mot *)
  for poss:=1 to length(mot) do
     (* Pour chaque ligne du caractre *)
     for ypol:=0 to 13 do begin
      temp:=police^[ord(mot[poss]),ypol];
      (* Pour chaque point de la ligne *)
      for xpol:=0 to 7 do begin
        (* Tester le bit *)
        if (temp and 128<>0) then begin
            (* Afficher le point et son ombre *)
            putpixel(page,x+(xpol+(poss-1)*8)+1,y+ypol+1,fcol);
            putpixel(page,x+(xpol+(poss-1)*8),y+ypol,tcol);
          end;
        (* Tester le bit suivant *)
        temp:=byte(temp shl 1);
      end;
    end;
end;

procedure wdelay(time:word);
(* La frquence de rafraichissement est de 70Hz en Mode 13h.
   Pour viter d'utiliser le DELAY de l'unit CRT (qui plante par ailleur sur
   gros processeur !), on peut attendre un certain nombre de rafraichissements.
   Par exemple 70 pour 1 seconde... *)
var cmpt:word;
begin
  for cmpt:=1 to time do waitretrace;
end;

end.